import os
import re
import subprocess

import click
from colorama import Back, Fore, Style

from .toolkit import Colored


class MkGit:
    _PATH_MAIN = subprocess.getoutput('git rev-parse --show-toplevel').strip()
    _PATH_MAIN_CONFIG = f'{_PATH_MAIN}/.git/config'
    _PATH_SUB_LS = subprocess.getoutput(
        "git config --file .gitmodules --get-regexp path | awk '{ print $2 }'"
    ).splitlines()
    _PATH_SUB = list()

    @classmethod
    def _get_submodules(cls):
        cls._PATH_SUB = [
            os.path.join(cls._PATH_MAIN, _) for _ in cls._PATH_SUB_LS
        ]

    @classmethod
    def _re_restores(cls, msg, restores):
        pattern = re.compile(r"(pathspec ')(.*)(')")
        for _ in pattern.findall(msg):
            restores.remove(_[1])

    @classmethod
    def add(cls, ignore):
        """ Auto add all files to git except submodules """
        cls._get_submodules()
        subprocess.run('git add .', shell=True)
        restores = ['git', 'restore', '--stage']

        if cls._PATH_SUB:
            restores.extend(cls._PATH_SUB_LS)

        if ignore:
            restores.extend(ignore)

        if len(restores) == 3:
            subprocess.run('git status', shell=True)
            return

        e = subprocess.run(restores, stderr=subprocess.PIPE, text=True).stderr
        if not e:
            subprocess.run('git status', shell=True)
            return

        cls._re_restores(msg=e, restores=restores)
        subprocess.run(restores)
        subprocess.run('git status', shell=True)
        print(e)

    @staticmethod
    def _re_words(info):
        """
        e.q.
            From https://xxx.net/xxx/xx
             - [
               group1: (deleted)
               group2: (]         (none)     -> origin/)
               group3: (feature/michael/xxxx)

             - [deleted] (none) -> origin/feature/xxxx/add-advanced-task
             - [deleted] (none) -> origin/patch/michael/rename-success
        """
        pattern = re.compile(r'\[(.+)(\].+-> origin/)(.+)')
        info = pattern.sub(
            rf'[{Fore.CYAN}\1{Fore.RESET}\2{Fore.CYAN}\3{Fore.RESET}', info)
        return info

    @staticmethod
    def _re_branchs(msg):
        """ """
        pattern_remote = re.compile(r'(remotes[/\w]*)')
        pattern_now = re.compile(r'(\*)( )(.*)')
        msg = pattern_remote.sub(rf'{Fore.RED}\1{Fore.RESET}', msg)
        msg = pattern_now.sub(
            rf'{Fore.YELLOW}\1{Fore.RESET}\2{Fore.GREEN}\3{Fore.RESET}', msg)
        return msg

    @classmethod
    def fetch(cls, show=True):
        """ sort out current branchs """
        prefix = Colored.get_color_prefix(color='LIGHTBLUE_EX',
                                          color_prefix=Fore.WHITE,
                                          prefix_msg='fetch')
        try:
            info = subprocess.getoutput('git fetch --prune')
        except Exception:
            print('No Git detect!')
            return

        msg = '--- Update ---' if info else 'Nothing To Update.'
        print(f'{prefix}{msg}')

        if not show and not info:
            return

        if 'deleted' in info:
            info = MkGit._re_words(info=info)
        print(info)

        # display branches
        click.echo('show branchs [y/N]: \n')
        show = click.getchar()
        if show == 'y':
            msg = cls._re_branchs(msg=subprocess.getoutput('git branch -a'))
            print(msg)

    @classmethod
    def _checkout(cls, branch):
        colored_repo = Colored.get_color_prefix(color='LIGHTBLACK_EX',
                                                color_prefix=Fore.WHITE)
        # """ check exist """
        branchs = subprocess.getoutput(
            "git for-each-ref "
            "--sort=-committerdate refs/heads/ "
            "--format='%(refname:short)'").splitlines()

        if branch in branchs:
            # """ checkout branch """
            info = subprocess.getoutput(f'git checkout {branch}')
            extra = ''
            try:
                if 'Already on' in info:
                    status = 'Already on '
                    color = Fore.CYAN

                elif 'Switched to' in info:
                    status = 'Switched to '
                    color = Fore.YELLOW

                else:
                    status = 'Error on '
                    color = Fore.RED
                    extra = f'\n{info}'

            except Exception:
                status = 'Except Error on '
                color = Fore.RED
                extra = f'\n{info}'

            print(
                f'{colored_repo}{status}{Colored.draw(color, branch)}{extra}'
            )
            return

        # """ create new branch """
        draw_branch = Colored.draw(Fore.CYAN, branch)
        msg = Colored.draw(Fore.RED, 'Not Exist')
        print(f'\n< Branch: {draw_branch} > {msg}.')

        click.echo(f'Do you wanna create < Branch: {draw_branch} > [y/N]:')
        create = click.getchar()
        if create == 'y':
            info = subprocess.getoutput(f'git checkout -b {branch}')
            if 'Switched to' in info:
                info = f'Switched to {Colored.draw(Fore.YELLOW, branch)}'
            print(f'{colored_repo}{info}')

    @classmethod
    def _current_branch(cls):
        colored_repo = Colored.get_color_prefix(color='LIGHTBLACK_EX',
                                                color_prefix=Fore.WHITE)
        output = None
        try:
            output = str(
                subprocess.check_output(['git', 'branch'],
                                        cwd=os.getcwd(),
                                        universal_newlines=True))
            branch = [a for a in output.split('\n') if a.find('*') >= 0][0]
            current_branch = branch[branch.find('*') + 2:]
            status = f'Stay on {Colored.draw(Fore.GREEN, current_branch)}'

        except Exception:
            status = f'Error on {Colored.draw(Fore.RED, "info")}\n{output}'

        print(f'{colored_repo}{status}')

    @classmethod
    def swap(cls, branch_name, ignore=list(), level='all'):
        """s branch_name
        swap current branch to target branch

        ignore: Ignore specific submodule.
        """
        if level == 'all' or level == '1':
            cls._checkout(branch=branch_name)

        if level == 'all' or level == '2':
            cls._get_submodules()
            if not cls._PATH_SUB:
                return
            for path in cls._PATH_SUB:
                sub_name = path.split('/')[-1]
                if sub_name in ignore:
                    cls._current_branch()
                    continue
                try:
                    os.chdir(path)
                except Exception:
                    continue
                cls._checkout(branch=branch_name)

    @ classmethod
    def _init(cls):
        if not os.path.exists(cls._PATH_MAIN_CONFIG):
            print('No Git Config Found!')
            return
        cls._get_submodules()
        if not cls._PATH_SUB:
            return

        has_files = None
        for path in cls._PATH_SUB:
            try:
                os.chdir(path)
                has_files = False if not subprocess.getoutput('ls') else True
            except Exception:
                return
        if has_files:
            return

        color_prefix = Colored.get_color_prefix(color='MAGENTA',
                                                color_prefix=Fore.WHITE,
                                                prefix_msg='init')
        print(f'{color_prefix}Git Submodeles')
        subprocess.run('git submodule update --init --recursive', shell=True)
        print(f'{color_prefix}Finished.\n')

    @ staticmethod
    def _re_pull_words(replacement, info, color):
        pattern = re.compile(rf'([{replacement}]+)([\n-)])')
        return pattern.sub(rf'{color}\1{Fore.RESET}\2', info)

    @ classmethod
    def _git_pull(cls):
        info = subprocess.getoutput('git pull')
        info = cls._re_pull_words(
            replacement='-', info=info, color=Fore.RED)
        info = cls._re_pull_words(
            replacement='+', info=info, color=Fore.GREEN)
        print(f'{info}\n')

    @ classmethod
    def pull(cls):
        """ pull all file from Git repo """
        cls._init()

        os.chdir(cls._PATH_MAIN)
        cls.fetch(show=False)
        cls._git_pull()

        if not cls._PATH_SUB:
            return
        for path in cls._PATH_SUB:
            try:
                os.chdir(path)
            except Exception:
                continue
            cls._git_pull()



"""
    | ┏┓       ┏┓
    ┏━┛┻━━━━━━━┛┻━┓
    ┃      ☃      ┃
    ┃  ┳┛     ┗┳  ┃
    ┃      ┻      ┃
    ┗━┓         ┏━┛
    | ┗┳        ┗━┓
    |  ┃          ┣┓
    |  ┃          ┏┛
    |  ┗┓┓┏━━━━┳┓┏┛
    |   ┃┫┫    ┃┫┫
    |   ┗┻┛    ┗┻┛
    God Bless,Never Bug.
"""
import click
from click_help_colors import HelpColorsCommand, HelpColorsGroup
from importlib_metadata import version

import mkie
from mkie.core.mkdk import Mkdk
from mkie.core.mkgit import MkGit


class Mkie(click.MultiCommand):

    @click.group(
        cls=HelpColorsGroup,
        help_headers_color='yellow',
        help_options_color='green',
        context_settings=dict(help_option_names=['-h', '--help']),
    )
    @click.version_option(version=version('mkie'), prog_name='mkie')
    def cli():
        """
        \b
                      __   _
           ____ ___  / /__(_)__
          / __ `__ \/ //_/ / _ \\
         / / / / / / ,< / /  __/
        /_/ /_/ /_/_/|_/_/\___/

        A useful tool for control clis in terminal.
        """
        pass

    @cli.command()
    @click.option('-i', '--ignore', help='ignore files', multiple=True)
    def gitadd(ignore):
        """ Auto add all files to git and ignore submodules. """
        MkGit.add(ignore=ignore)

    @cli.command()
    def gitfetch():
        """ sort out current branchs. """
        MkGit.fetch()

    @cli.command()
    @click.option('-i',
                  '--ignore',
                  help='ignore submodules',
                  is_flag=False,
                  flag_value='general',
                  multiple=True)
    @click.argument('branch_name', required=True)
    def s(ignore, branch_name):
        """ Swap current branch to target branch. """
        MkGit.swap(ignore=ignore, branch_name=branch_name)

    @cli.command()
    def gitpull():
        """ pull latest update from repo """
        MkGit.pull()

    @cli.command()
    @click.option('-f',
                  '--format',
                  help='pretty print container cols,'
                  'default:".ID.Names.Ports.Image"')
    @click.option('--pattern', help='rg pattern word container name')
    def dps(format, pattern):
        """ list docker containers """
        Mkdk.ps(format=format, pattern=pattern)

    @cli.command()
    def dbu():
        """ build docker container """
        Mkdk.build()

    @cli.command()
    @click.argument('project', required=False)
    def dup(project):
        """ start docker container """
        Mkdk.up(project=project)

    @cli.command()
    @click.argument('project', required=False)
    def dd(project):
        """ start docker container """
        Mkdk.down(project=project)


if __name__ == '__main__':
    Mkie.cli()



import os
import subprocess
from pathlib import Path

from colorama import Fore, Style

from .toolkit import Colored


class Mkdk:

    _DEFAULT_PS_FORMAT = 'table {{.ID}}\t{{.Names}}\t{{.Ports}}\t{{.Image}}'
    _PS_FORMAT = os.environ.get('MKIE_PS_FORMAT', _DEFAULT_PS_FORMAT)

    @classmethod
    def ps(cls, format, pattern):
        format = format or cls._PS_FORMAT
        _cmd = ['docker', 'ps', '--format', format]

        if pattern:
            _cmd.extend(['--filter', f'name={pattern}'])

        subprocess.run(_cmd)

    @classmethod
    def build(cls):
        subprocess.run(['docker-compose', 'build'])

    @classmethod
    def up(cls, project):
        _cmd = ['docker-compose',  'up', '-d']
        if project:
            _cmd.extend(['--project-name', project])

        project_name = project or Path.cwd().name
        color_prefix = Style.RESET_ALL + Fore.BLACK
        prefix = Colored.get_color_prefix(color='LIGHTYELLOW_EX',
                                          color_prefix=color_prefix,
                                          prefix_msg='🐳 Docker ⬆ ',
                                          bottom=True,
                                          bottom_color='LIGHTBLUE_EX',
                                          bottom_prefix=Fore.BLACK,
                                          bottom_msg=project_name)

        print(prefix)
        subprocess.run(_cmd)

    @classmethod
    def down(cls, project):
        _cmd = ['docker-compose',  'down']
        if project:
            _cmd.extend(['--project-name', project])

        project_name = project or Path.cwd().name
        color_prefix = Style.RESET_ALL + Fore.BLACK
        prefix = Colored.get_color_prefix(color='LIGHTGREEN_EX',
                                          color_prefix=color_prefix,
                                          prefix_msg='🐳 Docker ⬇ ',
                                          bottom=True,
                                          bottom_color='LIGHTBLUE_EX',
                                          bottom_prefix=Fore.BLACK,
                                          bottom_msg=project_name)

        print(prefix)
        subprocess.run(_cmd)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkie', 'mkie.core']

package_data = \
{'': ['*']}

install_requires = \
['click-help-colors==0.9.1',
 'click==8.0.3',
 'colorama>=0.4.6,<0.5.0',
 'importlib-metadata>=4.11.4,<5.0.0']

entry_points = \
{'console_scripts': ['mkie = mkie.__main__:Mkie.cli']}

setup_kwargs = {
    'name': 'mkie',
    'version': '0.1.6',
    'description': 'A useful tool for control clis in terminal.',
    'long_description': '# mkie\n\nA useful tool for control clis in terminal.\n\n## Installation\n\nIf you already know how to install python packages, then you can install it via pip.\n\n```\npip3 install mkie\n```\n\n## Upgrade\n\n```\npip3 install --upgrade mkie\n```\n\n## Features\n\n`mkie` is control clis:\n\n- Git\n  - `gitadd`: Auto add all files to git and ignore submodules.\n  - `gitfetch`: Sort out local branchs.\n  - `gitpull`: Pull latest update from git repo.\n  - `s`: Swap current branch to target branch.\n',
    'author': 'Michael Chou',
    'author_email': 'snoopy02m@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/cbb23021/mkie',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import os

from colorama import Back, Fore, Style


class Colored:

    _RESET = Style.RESET_ALL

    @classmethod
    def get_color_prefix(cls, color=None, color_prefix=None, prefix_msg=None,
                         bottom=False, bottom_color=None, bottom_prefix=None,
                         bottom_msg=None):
        """
        show prefix in terminal as below:

           zone1 zone2
           |    | |
            mkie

                   zone3
                  |     |
            mkie xxx 
        """
        # zone1
        color = color or 'BLACK'
        color_prefix = Style.BRIGHT + color_prefix + getattr(Back, color)
        prefix_msg = prefix_msg or os.path.basename(os.getcwd())
        color_edge = getattr(Fore, color, None)

        zone1 = f'\n{color_edge}{color_prefix}{prefix_msg}{cls._RESET}'

        if not bottom:
            zone2 = f'{color_edge}'
            return f'{zone1}{zone2}{cls._RESET} '

        # bottom
        bottom_color = bottom_color or 'BLACK'
        bottom_prefix = bottom_prefix + getattr(Back, bottom_color)
        bottom_edge = getattr(Fore, bottom_color, None)

        color_edge += getattr(Back, bottom_color)

        zone2 = f'{color_edge} '
        zone3 = f' {bottom_prefix}{bottom_msg} {cls._RESET}{bottom_edge}'

        return (f'{zone1}{zone2}{zone3}{cls._RESET}')

    @classmethod
    def draw(cls, color=None, msg=None):
        """
        colored msg only
        """
        return f'{color}{msg}{cls._RESET}'
